// Scanner Class

package java_cup.interpreter;

import java_cup.runtime.Symbol;

public class scanner implements java_cup.runtime.Scanner {
  
    /** the stream used to get the sequence of characters for parsing  */
    final java.io.InputStream instream;
    
    /** initialize the input stream */
    public scanner(java.io.InputStream is) throws java.io.IOException {
	instream = is;
    }
    public scanner() throws java.io.IOException { this(System.in); }
    
    /** single lookahead character */
    protected int next_char = -2;
    
    /** advance input by one character */
    protected void advance()
	throws java.io.IOException
    { next_char = instream.read(); }
    
    /** initialize the scanner */
    private void init()
	throws java.io.IOException
    { advance(); }
    
    /** recognize and return the next complete token */
    public Symbol next_token()
	throws java.io.IOException
    {
	if (next_char==-2) init(); // set stuff up first time we are called.
	for (;;)
	    switch (next_char)
		{
		case 'a': case 'b': case 'c': case 'd': case 'e': case 'w':
		case 'f': case 'g': case 'h': case 'i': case 'j': case 'E':
		case 'k': case 'l': case 'm': case 'n': case 'o': case 'O':
		case 'p': case 'q': case 'r': case 's': case 't': case 'W':
		case 'z': case 'u': case 'v': case 'x': case 'M': case 'y':
		case 'P': case 'B': 
		    /* parsing a method name, type variable or a location variable */
		    
		    /* in this simple scanner a variable is made of lowercase letters and of two uppercases letters: O, M. These 
		     uppercased letters are used to recognize tokens as Object and addMethod */
		    String name = "";
	 
		    do{
			name = name + (char)next_char;
			
			if(name.compareTo("clone") == 0){
			    //System.out.println("Operation: clone"); 
			    advance(); return new Symbol(sym.CLONE);}
			else if(name.compareTo("delegate") == 0){
			    //System.out.println("Operation: delegate"); 
			    advance(); return new Symbol(sym.DELEGATE);}
			else if(name.compareTo("addMethod") == 0){
			    //System.out.println("Operation: addMethod"); 
			    advance();return new Symbol(sym.ADDMETHOD);}
			else if(name.compareTo("apply") == 0){
			    //System.out.println("Operation: apply"); 
			    advance(); return new Symbol(sym.APPLY);}
			else if(name.compareTo("super") == 0){
			    //System.out.println("Symbol: super"); 
			    advance(); return new Symbol(sym.SUPER);}
			else if(name.compareTo("change") == 0){
			    //System.out.println("Operation: change"); 
			    advance(); return new Symbol(sym.CHANGE);}
			else if(name.compareTo("linearity") == 0){
			    //System.out.println("linearity"); 
			    advance(); return new Symbol(sym.LINEARITY);}
			else if(name.compareTo("fn") == 0){
			    //System.out.println("Symbol: fn"); 
			    advance(); return new Symbol(sym.FN);}
			advance();
		    }while(next_char != '.' & next_char != '(' & next_char != ')' & next_char != ',' & next_char != ':' & next_char != ' ' & next_char != '\n' & next_char != '<' & next_char != '>' & next_char != '|' & next_char != ';');
		    //System.out.println("var: "+name+"@");

		    /* String recognized as variable */
		    return new Symbol(sym.VAR, name);
		    
	      case '0': case '1': case '2': case '3': case '4': 
	      case '5': case '6': case '7': case '8': case '9': 
	      /* parse a decimal integer */
		    int i_val = 0;
	      do {
	        i_val = i_val * 10 + (next_char - '0');
	        advance();
	      } while (next_char >= '0' && next_char <= '9');
		String num = ""+i_val;
		//System.out.println("var: "+num+"@");
	      return new Symbol(sym.VAR, num);
		    
		case '.': advance(); /*System.out.println("DOT");*/ return new Symbol(sym.DOT);
		case ',': advance(); /*System.out.println("COMMA");*/return new Symbol(sym.COMMA);
		case '_': advance(); /*System.out.println("UNDERSCORE");*/return new Symbol(sym.UNDERSCORE);
		case ':': advance(); /*System.out.println("COLON");*/return new Symbol(sym.COLON);
		case '<': advance(); /*System.out.println("LANGLE");*/return new Symbol(sym.LANGLE);
		case '>': advance(); /*System.out.println("RANGLE");*/return new Symbol(sym.RANGLE);
		case '(': advance(); /*System.out.println("LPAREN");*/return new Symbol(sym.LPAREN);
		case ')': advance(); /*System.out.println("RPAREN");*/return new Symbol(sym.RPAREN);
		case '*': advance(); /*System.out.println("BANG");*/ return new Symbol(sym.BANG);
		case '|': advance(); /*System.out.println("LINE");*/ return new Symbol(sym.LINE);
		case ';': advance(); /*System.out.println("SEMICOLON");*/ return new Symbol(sym.SEMICOLON);
		case '-': advance(); if( next_char == '>' ) { advance(); /*System.out.println("ARRAY");*/ return new Symbol(sym.ARRAY);} else if(next_char == 'o'){advance();/*System.out.println("LINEAR");*/ return new Symbol(sym.LINEAR);} else return new Symbol(sym.EOF);    
		case '\n': return new Symbol(sym.EOF);
		    
		default: 
		    /* in this simple scanner we just ignore everything else */
		    advance();
		    break;
		}
    }
}
